#include "main.h"

#define MissileColor SDL_MapRGB(Screen->format,0,200,0)
#define MISSILE_MAX_NUM 100
#define MISSILE_LENGTH 200
#define TAIL 80

typedef struct
{
    float x,y;
    float xspeed,yspeed;
    coord Pixels[MISSILE_LENGTH];
    int CurrentPixel;
    int Proceeded;
    float Erase;
    int Target;
    int TargetX;
    int Split;
} MissileType;

MissileType Missile[MISSILE_MAX_NUM];
int MissileCount=0;

void CreateMissile(int x, int y)
{
    int i;
    
    if (MissileCount<MISSILE_MAX_NUM) {

        MissileType *This;
	float Vec;

        This=&Missile[MissileCount];
	This->x=x;
        This->y=y;
	This->Target=abrand(0,8);
	if (This->Target<3)
	    This->TargetX = 120+This->Target*100;
	else if (This->Target<6)
	    This->TargetX = 480+(This->Target-3)*100;
	else 
	    This->TargetX = 40+(This->Target-6)*360;
	Vec=100*hypot(This->TargetX-This->x,510-This->y);
	This->xspeed=Wave[WaveNum].MissileSpeed*(This->TargetX-This->x)/Vec;
        This->yspeed=Wave[WaveNum].MissileSpeed*(510-This->y)/Vec;
        This->CurrentPixel=0;
        This->Erase=0;
        This->Split=0;
	for (i=0;i<MISSILE_LENGTH;i++) 
	    {This->Pixels[i].x=0;This->Pixels[i].y=0;}
	MissileCount++;
    }
    else printf("Too many Missiles\n");
}

void RemovePix(int i, int a)
{
    SDL_Rect rect;
    int DelPix;

    rect.w=1;
    rect.h=1;
    DelPix=(Missile[i].CurrentPixel+a) % MISSILE_LENGTH;
    rect.x=Missile[i].Pixels[DelPix].x;
    rect.y=Missile[i].Pixels[DelPix].y;
    PutPixel(Screen,rect.x,rect.y,MissileColor);
    SDL_BlitSurface(BackBuffer, &rect, Screen, &rect);
    AddThisRect(rect);
}

void RemoveMissile(int i)
{
    int a,b;

    Missile[i].Erase+=Speed/(float)3;
    if (Missile[i].Erase>255) {
	SDL_SetAlpha(BackBuffer,0,0);
 	for (a=0;a<MISSILE_LENGTH;a++) RemovePix(i,a);
	Missile[i]=Missile[MissileCount-1];
	MissileCount--;
    } else {
	for (a=1;a<TAIL;a++)
	{
	    b=(a/(float)(TAIL+1)*(float)256)-Missile[i].Erase;
	    if (b<0) b=0;
	    SDL_SetAlpha(BackBuffer,0,0);  // alpha bug fix
	    SDL_SetAlpha(BackBuffer,SDL_SRCALPHA,255-b);
	    RemovePix(i,a);
	}
	SDL_SetAlpha(BackBuffer,SDL_SRCALPHA, (Missile[i].Erase));
	for (a=TAIL;a<MISSILE_LENGTH;a++)
	    RemovePix(i,a);
    }
}

void DestroyTarget(int TargetNum, int TargetX)
{
    SDL_Rect rect;

    if (TargetNum<6) {
	rect.x=TargetX-50;
	rect.y=490;
	rect.w=100;
	rect.h=CityPic->h;
	SDL_BlitSurface(BackPic,&rect,BackBuffer,&rect);
	BlitToBB(rect.x,rect.y,CityHitPic);
	SDL_SetAlpha(BackBuffer,0,0);
	BlitPart(rect.x,rect.y,BackBuffer,rect);
	if ( ! CityDestroyed[TargetNum] ) {
	    CityDestroyed[TargetNum]=1;
	    CityNum--;
	}
    } else if (TargetNum<9) {
	rect.x=TargetX-CannonHitPic->w/2;
	rect.y=510;
	rect.w=CannonHitPic->w;
	rect.h=CannonHitPic->h;
	SDL_BlitSurface(BackPic,&rect,BackBuffer,&rect);
	BlitToBB(rect.x,rect.y,CannonHitPic);
	SDL_SetAlpha(BackBuffer,0,0);
	BlitPart(rect.x,rect.y,BackBuffer,rect);
	CannonDestroyed[TargetNum-6]=1;
	ShotNum[TargetNum-6]=-1;
	DrawShotNum(TargetNum-6);
    } else printf("WARNING: Target was %d!\n",TargetNum);
}

void HandleMissile()
{
    int i;
    
    for (i=0;i<MissileCount;i++)
    {
	// Hit!
	if ((Missile[i].y>=510)&&(Missile[i].Erase==0)) {
	    CreateExplosion(Missile[i].x,Missile[i].y);
	    DestroyTarget(Missile[i].Target,Missile[i].TargetX);
	    PlaySound(MissileExplosionSound);
	}
	// Delete Missile
	if  (
		((Missile[i].y>=510)
		||((int)Missile[i].x<0)
		||((int)Missile[i].x>=800))
		||(Missile[i].Erase>0)
	    )
	    RemoveMissile(i);
	else {
	    // Move Missile
	    Missile[i].x+=Missile[i].xspeed*Speed;
    	    Missile[i].y+=Missile[i].yspeed*Speed;
	    // Split Missile (Mirv)
	    if (Missile[i].Split) {
	        CreateMissile(Missile[i].x,Missile[i].y);
		Missile[i].Split=0;
	    }
	}
    }
}

void DrawMissile()
{
    int i,a,DrawPix;
    SDL_Rect rect;
    
    lock();
    // Calculate next point
    for (i=0;i<MissileCount;i++)
    if (Missile[i].Erase==0)
    if (
	    ((int)Missile[i].x!=Missile[i].Pixels[Missile[i].CurrentPixel].x)
	    ||((int)Missile[i].y!=Missile[i].Pixels[Missile[i].CurrentPixel].y)	    
	)
    {
	//printf("draw %d - %d \n\t%d - %d\n",(int)Missile[i].x,Missile[i].Pixels[Missile[i].CurrentPixel].x,(int)Missile[i].y,Missile[i].Pixels[Missile[i].CurrentPixel].y);
	if (++Missile[i].CurrentPixel==MISSILE_LENGTH)
	    Missile[i].CurrentPixel=0;
	Missile[i].Pixels[Missile[i].CurrentPixel].x=Missile[i].x;
	Missile[i].Pixels[Missile[i].CurrentPixel].y=Missile[i].y;
	Missile[i].Proceeded=1;
    }
    else Missile[i].Proceeded=0;
    // Draw Missile
    rect.w=1;
    rect.h=1;
    for (i=0;i<MissileCount;i++)
    {
	// Draw Tail
	if (Missile[i].Erase==0)
        for (a=1;a<TAIL;a++) {
    	    SDL_SetAlpha(BackBuffer,SDL_SRCALPHA, 255-a/(float)(TAIL+1)*(float)256);
	    RemovePix(i,a);
        }
	// Draw Body
	if (Missile[i].Erase==0)
        for (a=TAIL;a<MISSILE_LENGTH;a++) {
	    DrawPix=(Missile[i].CurrentPixel+a) % MISSILE_LENGTH;
	    PutPixel(Screen,
		    Missile[i].Pixels[DrawPix].x,
		    Missile[i].Pixels[DrawPix].y,
		    MissileColor);
	}
        SDL_SetAlpha(BackBuffer,0, 0);
        RemovePix(i,1);
    }	
    unlock();
}

void UndrawMissile()
{
}

void MissileCollision(int x, int y, int rad)
{
    int i;

//    printf("ads %d - %d - %d\n",x,y,rad);
    for (i=0;i<MissileCount;i++)
    {
	if (Missile[i].Erase==0)
	if (hypot(Missile[i].x-x,Missile[i].y-y)<rad) {
	    CreateExplosion(Missile[i].x,Missile[i].y);
	    RemoveMissile(i);
	    AddScore(20);
	    PlaySound(MissileExplosionSound);
	}
    }
}

int Split(int MissileNum)
{
    if (Missile[MissileNum].y>400) return 1;
    Missile[MissileNum].Split=1; return 0;
}
